#!/bin/bash

set -e
set -o pipefail

argc="$#"
argv=("$@")

script_library_directory="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

while getopts ":-:s:r:p:" optchar
do
	case "${optchar}" in
		-)
			case "${OPTARG}" in
				sourceRoot)
					srcroot="${!OPTIND}"
					OPTIND=$(( $OPTIND + 1 ))
					;;
				releaseCandidate)
					release_candidate="${!OPTIND}"
					OPTIND=$(( $OPTIND + 1 ))
					;;
				projectFilename)
					project_filename="${!OPTIND}"
					OPTIND=$(( $OPTIND + 1 ))
					;;
				*)
					if [ "${OPTERR}" = 1 ] && [ "${optspec:0:1}" != ":" ]
					then
						echo "unknown option --${OPTARG}" >&2
						unknown_options=1
					fi
					;;
			esac
			;;
		s)
			srcroot="${OPTARG}"
			;;
		r)
			release_candidate="${OPTARG}"
			;;
		p)
			project_filename="${OPTARG}"
			;;
		*)
			if [ "${OPTERR}" = 1 ] && [ "${optspec:0:1}" != ":" ]
			then
				echo "unknown option --${OPTARG}" >&2
				unknown_options=1
			fi
			;;
	esac
done

declare -a schemes

if [ "${argv[$OPTIND-2]}" = "--" ]
then
	scheme_argument_index=$(( $OPTIND - 1 ))
	scheme_index=0
	
	while (( (scheme_argument_index + scheme_index) < $argc ))
	do
		current_argument_index=$(( scheme_argument_index + scheme_index ))
		schemes[$scheme_index]="${argv[$current_argument_index]}"
		
		scheme_index=$(( scheme_index + 1 ))
	done
fi

if [ ! -d "${srcroot}" ]
then
	echo "--sourceRoot \"${srcroot}\" must exist and be a directory"
	exit -1
fi

fixtures_directory="$( cd "${srcroot}" && pwd )/Externals/Scripts"
if [ ! -d "${fixtures_directory}" ]
then
	echo "couldn't find fixtures directory in --sourceRoot"
	exit -1
fi

if [ -z "${release_candidate}" ]
then
	echo "--releaseCandidate must not be empty"
	exit -1
fi
if [ ! -z $( echo "${release_candidate}" | tr -d 'rc[:digit:]' ) ]
then
	echo "--releaseCandidate must be of the form rcX; where X is any number of digit characters"
	exit -1
fi

if [ ! -z "${unknown_options}" ]
then
	exit -1
fi

build_temporary_directory="/tmp/com.realmacsoftware.dts.build"

if [ ! -d "${build_temporary_directory}" ]
then
	mkdir "${build_temporary_directory}"
fi

date_suffix=$(date "+%Y-%m-%d")
#
# This could be based on a user default read using x-man-page://defaults but the fixed directory works for me
# If someone else starts using this script we can make it default based
#
output_directory="${HOME}/Projects/Local Archives/${date_suffix}"

if [ ! -d "${output_directory}" ]
then
	mkdir -p "${output_directory}"
fi

# Utility Functions

function scheme_build {
	scheme=$1
	archive_prefix=$2
	archive_suffix=$3
	project_filename=$4
	
	#
	
	echo -e "Building scheme \"${scheme}\"" "\n"
	
	# build archive
	
	temporary_directory=$(mktemp -d "${build_temporary_directory}/XXXXXXXXXX")
	
	project_argument=""
	if [ ! -z "${project_filename}" ]
	then
		project_argument="-project ${project_filename}"
	fi
	
	xcodebuild -scheme "${scheme}" ${project_argument} archive "-IDECustomDistributionArchivesLocation=${temporary_directory}"
	
	# find archive in TEMPORARY_DIRECTORY
	
	temporary_archive_path=$(find "${temporary_directory}" -name '*.xcarchive')
	if [ -z "${temporary_archive_path}" ]
	then
		echo -e "Building scheme \"${scheme}\" failed, aborting" "\n"
		exit 1
	fi
	
	temporary_archive_infoplist_path="${temporary_archive_path}/Info.plist"
	
	plistbuddy_path="/usr/libexec/PlistBuddy"
	temporary_archive_application_product_path=$("${plistbuddy_path}" -c Print:ApplicationProperties:ApplicationPath "${temporary_archive_infoplist_path}")
	temporary_archive_application_path="${temporary_archive_path}/Products/${temporary_archive_application_product_path}"

	temporary_archive_application_infoplist_path="${temporary_archive_application_path}/Contents/Info.plist"
	if [ ! -f "${temporary_archive_application_infoplist_path}" ]
	then
		temporary_archive_application_infoplist_path="${temporary_archive_application_path}/Info.plist"
	fi
	
	# read values from the application Info.plist
	
	version_bundle_number=$("${plistbuddy_path}" -c Print:CFBundleVersion "${temporary_archive_application_infoplist_path}")
	version_marketing_number=$("${plistbuddy_path}" -c Print:CFBundleShortVersionString "${temporary_archive_application_infoplist_path}")
	
	# determine what to call this archive based on the standard naming scheme
	
	destination_archive_name="${archive_prefix}_${version_bundle_number}_${version_marketing_number}${release_candidate}-${archive_suffix}.xcarchive"
	new_temporary_archive_path="${temporary_directory}/${destination_archive_name}"
	mv "${temporary_archive_path}" "${new_temporary_archive_path}"
	
	# validate the archive
	
	echo -e "Validating scheme \"${scheme}\"" "\n"
	"${script_library_directory}/xcarchive_check.rb" -archivePath "${new_temporary_archive_path}" -fixturesPath "${fixtures_directory}"
	if [ $? -ne 0 ]
	then
		echo -e "Validating scheme \"${scheme}\" failed, aborting" "\n"
		exit 1
	fi
	
	# move archive to the output directory
	
	destination_archive_path="${output_directory}/${destination_archive_name}"
	if [ -e "${destination_archive_path}" ]
	then
		echo -e "archive already exists at ${destination_archive_path}, aborting" "\n"
		exit 1
	fi
	
	ditto "${new_temporary_archive_path}" "${destination_archive_path}"
	
	rm -r "${temporary_directory}"
}

function lowercase {
	tr '[:upper:]' '[:lower:]'
}

function strip_whitespace {
	tr -d ' '
}

function main {
	cd "${srcroot}"
	
	# Build and check each scheme, then copy the the archive directory
	
	for current_scheme in "${schemes[@]}"
	do
		scheme="${current_scheme}"
		
		archive_prefixsuffix=$(printf %s "${current_scheme}" | lowercase | strip_whitespace)
		
		archive_prefix="${archive_prefixsuffix%-*}"
		
		archive_suffix="${archive_prefixsuffix#*-}"
		if [ "${archive_prefix}" = "${archive_suffix}" ]
		then
			archive_suffix="debug"
		fi
		
		scheme_build "${scheme}" "${archive_prefix}" "${archive_suffix}" "${project_filename}"
	done
}

/bin/bash "${script_library_directory}/workingcopy_check.sh" --sourceRoot "${srcroot}"
if [ ! $? -eq 0 ]
then
	echo -e "workingcopy_check.sh failed, aborting"
	exit 1
fi
echo -e ""

main
